@@ -1,11 +1,12 @@
 # -*- coding: utf-8 -*-
+# cython: language_level=3
 
-## Copyright 1999-2012 by LivingLogic AG, Bayreuth/Germany
-## Copyright 1999-2012 by Walter Dörwald
+## Copyright 1999-2014 by LivingLogic AG, Bayreuth/Germany
+## Copyright 1999-2014 by Walter Dörwald
 ##
 ## All Rights Reserved
 ##
-## See ll/__init__.py for the license
+## See ll/xist/__init__.py for the license
 
 
 """
@@ -51,13 +52,13 @@ def getdoc(thing, format):
 	text = "\n".join(lines)
 
 	if inspect.ismethod(thing):
-		fmt = "METHOD-DOCSTRING({0}.{1.__class__.__name__}.{1.__name__})"
+		fmt = "METHOD-DOCSTRING({0}.{1.__class__.__name__}.{1.__qualname__})"
 	elif isinstance(thing, property):
 		fmt = "PROPERTY-DOCSTRING({0}.{1})"
 	elif inspect.isfunction(thing):
-		fmt = "FUNCTION-DOCSTRING({0}.{1.__name__})"
+		fmt = "FUNCTION-DOCSTRING({0}.{1.__qualname__})"
 	elif inspect.isclass(thing):
-		fmt = "CLASS-DOCSTRING({0}.{1.__name__})"
+		fmt = "CLASS-DOCSTRING({0}.{1.__qualname__})"
 	elif inspect.ismodule(thing):
 		fmt = "MODULE-DOCSTRING({0})"
 	else:
@@ -134,36 +135,44 @@ def _namekey(obj, name):
 
 
 def _codeheader(thing, name, type):
-	(args, varargs, varkw, defaults) = inspect.getargspec(thing)
+	# FullArgSpec(args, varargs, varkw, defaults, kwonlyargs, kwonlydefaults, annotations)
+	spec = inspect.getfullargspec(thing)
+	# kwonlyargs, kwonlydefaults
 	sig = xsc.Frag()
-	offset = len(args)
-	if defaults is not None:
-		offset -= len(defaults)
-	for i in range(len(args)):
+	offset = len(spec.args)
+	if spec.defaults is not None:
+		offset -= len(spec.defaults)
+	for (i, arg) in enumerate(spec.args):
 		if i == 0:
 			if issubclass(type, meth):
-				if args[i] == "self":
-					sig.append(var(self()))
-				elif args[i] == "cls":
-					sig.append(var(cls()))
+				if arg == "self":
+					sig.append(obj(self()))
+				elif arg == "cls":
+					sig.append(obj(cls()))
 				else:
-					sig.append(var(args[i]))
+					sig.append(obj(arg))
 			else:
-				sig.append(var(args[i]))
+				sig.append(obj(arg))
 		else:
 			if sig:
 				sig.append(", ")
-			sig.append(var(args[i]))
+			sig.append(obj(arg))
 		if i >= offset:
-			sig.append("=", lit(repr(defaults[i-offset])))
-	if varargs:
+			sig.append("=", lit(repr(spec.defaults[i-offset])))
+	if spec.varargs:
 		if sig:
 			sig.append(", ")
-		sig.append("*", var(varargs))
-	if varkw:
+		sig.append("*", obj(spec.varargs))
+	if spec.varkw:
 		if sig:
 			sig.append(", ")
-		sig.append("**", var(varkw))
+		sig.append("**", obj(spec.varkw))
+	for (i, arg) in enumerate(spec.kwonlyargs):
+		if sig:
+			sig.append(", ")
+		sig.append(obj(arg))
+		if arg in spec.kwonlydefaults:
+			sig.append("=", lit(repr(spec.kwonlydefaults[arg])))
 	sig.insert(0, type(name), "\u200b(") # use "ZERO WIDTH SPACE" to allow linebreaks
 	sig.append(")")
 	return sig
@@ -183,16 +192,16 @@ class _stack(object):
 
 def explain(thing, name=None, format=None, context=[]):
 	"""
-	Return a XML representation of the doc string of :var:`thing`, which can be
+	Return a XML representation of the doc string of :obj:`thing`, which can be
 	a function, method, class or module.
 
-	If :var:`thing` is not a module, you must pass the context in :var:`context`,
-	i.e. a list of names of objects into which :var:`thing` is nested. This
+	If :obj:`thing` is not a module, you must pass the context in :obj:`context`,
+	i.e. a list of names of objects into which :obj:`thing` is nested. This
 	means the first entry will always be the module name, and the other entries
 	will be class names.
 	"""
 
-	def _append(all, obj, var):
+	def _append(all, obj, varname):
 		try:
 			all.append((_namekey(obj, varname), obj, varname))
 		except (IOError, TypeError):
@@ -263,7 +272,7 @@ def explain(thing, name=None, format=None, context=[]):
 					ref = class_(baseclass.__name__)
 				else:
 					try:
-						baseclassname = baseclass.__fullname__
+						baseclassname = baseclass.__qualname__
 					except AttributeError:
 						baseclassname = baseclass.__name__
 					if thing.__module__ != baseclass.__module__:
@@ -292,7 +301,7 @@ def explain(thing, name=None, format=None, context=[]):
 		all = []
 		properties = []
 		classes = []
-		for varname in thing.__dict__.keys():
+		for varname in thing.__dict__:
 			obj = getattr(thing, varname)
 			if isinstance(obj, property):
 				properties.append((obj, varname))
@@ -304,10 +313,10 @@ def explain(thing, name=None, format=None, context=[]):
 				else:
 					classes.append((obj, varname))
 					_append(all, obj, varname)
-			elif inspect.ismethod(obj):
+			elif inspect.isfunction(obj):
 				# skip the method if it's a property getter, setter or deleter
 				for (prop, name) in properties:
-					if obj.__func__==prop.fget or obj.__func__==prop.fset or obj.__func__==prop.fdel:
+					if obj==prop.fget or obj==prop.fset or obj==prop.fdel:
 						break
 				else:
 					_append(all, obj, varname)
@@ -810,15 +819,15 @@ class cls(inline):
 		return e.convert(converter)
 
 
-class var(code):
+class obj(code):
 	"""
-	The name of a function or method argument.
+	A object of unspecified type.
 	"""
 	xmlns = xmlns
 	model = sims.ElementsOrText(rep, self, cls)
 
 	def convert_docbook(self, converter):
-		e = converter.target.parameter(self.content)
+		e = converter.target.varname(self.content)
 		return e.convert(converter)
 
 
@@ -1506,10 +1515,10 @@ class pyref(inline):
 			prop = str(self.attrs.property.convert(converter))
 		else:
 			prop = None
-		if "class_" in self.attrs:
-			class__ = str(self.attrs.class_.convert(converter)).replace(".", "-")
+		if "class" in self.attrs:
+			class_ = str(self.attrs.class_.convert(converter)).replace(".", "-")
 		else:
-			class__ = None
+			class_ = None
 		if "module" in self.attrs:
 			module = str(self.attrs.module.convert(converter))
 			if module.startswith("ll."):
@@ -1527,14 +1536,14 @@ class pyref(inline):
 				if module is not None:
 					e = target.a(e, href=(context.base, module, "/index.html#", function))
 			elif method is not None:
-				if class__ is not None and module is not None:
-					e = target.a(e, href=(context.base, module, "/index.html#", class__, "-", method))
+				if class_ is not None and module is not None:
+					e = target.a(e, href=(context.base, module, "/index.html#", class_, "-", method))
 			elif prop is not None:
-				if class__ is not None and module is not None:
-					e = target.a(e, href=(context.base, module, "/index.html#", class__, "-", prop))
-			elif class__ is not None:
+				if class_ is not None and module is not None:
+					e = target.a(e, href=(context.base, module, "/index.html#", class_, "-", prop))
+			elif class_ is not None:
 				if module is not None:
-					e = target.a(e, href=(context.base, module, "/index.html#", class__))
+					e = target.a(e, href=(context.base, module, "/index.html#", class_))
 			elif module is not None:
 				e = target.a(e, href=(context.base, module, "/index.html"))
 		return e.convert(converter)
