@@ -1,8 +1,9 @@
 #! /usr/bin/env python
 # -*- coding: utf-8 -*-
+# cython: language_level=3
 
-## Copyright 1999-2012 by LivingLogic AG, Bayreuth/Germany
-## Copyright 1999-2012 by Walter Dörwald
+## Copyright 1999-2014 by LivingLogic AG, Bayreuth/Germany
+## Copyright 1999-2014 by Walter Dörwald
 ##
 ## All Rights Reserved
 ##
@@ -134,7 +135,7 @@ def getxmlns(dtd):
 
 
 def adddtd2xnd(ns, dtd):
-	# Appends DTD information from :var:`dtd` to the :class:`xnd.Module` object
+	# Appends DTD information from :obj:`dtd` to the :class:`xnd.Module` object
 	from lxml import etree # This requires lxml (http://lxml.de/)
 	dtd = etree.DTD(dtd)
 
@@ -149,7 +150,7 @@ def adddtd2xnd(ns, dtd):
 	# Add element info
 	elements = sorted(dtd.iterelements(), key=namegetter)
 	for elemdecl in elements:
-		e = xnd.Element(elemdecl.name, xmlns=xmlns)
+		e = xnd.Element(xmlns, elemdecl.name)
 
 		# Add attribute info for this element
 		attrs = sorted(elemdecl.iterattributes(), key=namegetter)
@@ -194,7 +195,7 @@ def adddtd2xnd(ns, dtd):
 				return content
 			elementcontent = extractcont(elemdecl.content)
 			if elementcontent:
-				modelargs = [ns.elements[(name, xmlns)] for name in elementcontent]
+				modelargs = [ns.elements[(xmlns, name)] for name in elementcontent]
 				if elemdecl.type == "mixed":
 					modeltype = "sims.ElementsOrText"
 				else:
@@ -205,7 +206,7 @@ def adddtd2xnd(ns, dtd):
 					modeltype = "sims.NoElements"
 				else:
 					modeltype = "sims.NoElementsOrText"
-		e = ns.elements[(elemdecl.name, xmlns)]
+		e = ns.elements[(xmlns, elemdecl.name)]
 		if ns.model == "simple":
 			modeltype = modeltype == "sims.Empty"
 			modelargs = None
